/*		 ICP-PCI Kernel module free public release
 *
 *		       Copyright (C) 1996-99 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: icpciIOP.c,v 1.1.2.2 1999/07/21 08:41:56 matthias Exp $
 *
 */

/*
 * Defines:
 */

/*
 * Includes:
 */

#include "icpciOS.h"	/* os specific stuff */
#include "icpciExt.h"	/* offical driver interface */
#include "icpciInt.h"	/* driver internals */
#include "icpciReg.h"	/* register definitions IC-PCI */

/*
 * Globals
 */

extern struct 
icp_global_t icp;	/* main driver structure */

/*
 * Prototypes
 */

/*
 * icp_iop_open
 */
int
icp_iop_open (int board, icp_flags_t flags)
{

  ICP_PRINTF("icpci[%c]: icp_iop_open.\n", 'F');

  if (((flags & O_ACCMODE) == O_WRONLY) || ((flags & O_ACCMODE) == O_RDWR))
    {
      if (icp.write_lock[board])
	return -EBUSY;
      /* write: make sure, that the output port strobe pin is high */
      if (cr_set_outstrb (board, HIGH))
	{
	  icp_error (board, "cr_set_outstrb");
	  return -EIO;
	}
      icp.write_lock[board] = TRUE;
    }

  if (((flags & O_ACCMODE) == O_RDONLY) || ((flags & O_ACCMODE) == O_RDWR))
    {
      if (icp.read_lock[board])
	return -EBUSY;
      /* read: distinguish between raw or latched mode (with interrupts) */
      if (flags & O_NONBLOCK)
	{
	  /* set to raw mode */
	  if (cr_set_inregenb (board, CR_DISABLE))
	    {
	      icp_error (board, "cr_set_inregenb");
	      return -EIO;
	    }
	}
      else
	{
	  /* set to latched mode */
	  if (cr_set_inregenb (board, CR_ENABLE))
	    {
	      icp_error (board, "cr_set_inregenb");
	      return -EIO;
	    }
	  /* clear the input port buffer */
	  if (cr_set_inregclr (board, CR_ENABLE))
	    {
	      icp_error (board, "cr_set_inregclr");
	      return -EIO;
	    }
	  if (cr_set_inregclr (board, CR_DISABLE))
	    {
	      icp_error (board, "cr_set_inregclr");
	      return -EIO;
	    }
	  /* we expect a h-l edge at the input port strobe pin */
	  if (cr_set_instrbpol (board, CR_INSTRB_ACT_LOW))
	    {
	      icp_error (board, "cr_set_instrbpol");
	      return -EIO;
	    }
	  /* the interrupt pin should be connect with the strobe pin */
	  if (cr_set_inportpol (board, CR_INPORT_ACT_LOW))
	    {
	      icp_error (board, "cr_set_inportpol");
	      return -EIO;
	    }
	}
      /* enable some useful interrupts */
      icp_enable_irq (board, ICR_MINT | CR_INPINT);
      icp.read_lock[board] = TRUE;
      /* reset statistics */
      icp.acc[board].events_captured = 0UL;
      icp.acc[board].bytes_valid = 0UL;
      icp.acc[board].bytes_lost = 0UL;
    }

  return OK;
}

/*
 * icp_iop_close
 */
int
icp_iop_close (int board, icp_flags_t flags)
{

  ICP_PRINTF("icpci[%c]: icp_iop_close.\n", 'F');

  /* disable used interrupts */
  icp_disable_irq (board, ICR_MINT | CR_INPINT);
  /* disable used timers */
  icp_del_timer (board, IOPORT);

  if (flags & O_WRONLY || flags & O_RDWR)
    icp.write_lock[board] = FALSE;
  if (flags & O_RDONLY || flags & O_RDWR)
    icp.read_lock[board] = FALSE;
  
  return OK;
}

/*
 * icp_iop_read
 */
icp_ssize_t
icp_iop_read (int board, icp_flags_t flags, char *buffer, icp_size_t count)
{

  ICP_PRINTF("icpci[%c]: icp_icp_iop_read.\n", 'F');

  icp.inp_buffer[board] = icp_malloc (count, PAGED);
  icp.inp_count[board] = count;
  icp. inp_index[board] = 0;

  /* distinguish between raw or latched mode (with interrupts) */
  if (flags & O_NONBLOCK)
    while (icp.inp_index[board] < icp.inp_count[board])
      icp.inp_buffer[board][icp.inp_index[board]++] = cr_get_inport (board);
  else
    {
      icp.timeout[board] = FALSE;
      icp_sleep (board, ICP_WAIT_IOP, IOPORT);
      if (icp.timeout[board])
	return -ENODATA;
    }
  
  icp_copyout (buffer, icp.inp_buffer[board], icp.inp_index[board]);
  icp_free (icp.inp_buffer[board], PAGED);
  icp.inp_buffer[board] = NULL;
  icp.inp_count[board] = 0;
  
  return icp.inp_index[board];
}

/*
 * icp_iop_write
 */
int
icp_iop_write (int board, const char *buffer, icp_size_t count)
{
  char *outp_buffer = NULL;
  icp_size_t outp_count = 0;
  icp_ssize_t outp_index = 0;

  ICP_PRINTF("icpci[%c]: icp_iop_write.\n", 'F');

  outp_buffer = icp_malloc (count, PAGED);
  outp_count = count;
  outp_index = 0;

  icp_copyin (outp_buffer, buffer, outp_count);

  /* write the all data as fast as possible to the outport */
  /* later we should implement a adjustable data rate here */
  while (outp_index < outp_count)
    {
      if (cr_set_outport (board, outp_buffer[outp_index++]))
        {
	  icp_error (board, "cr_set_outport");
	  icp_free (outp_buffer, PAGED);
	  return -EIO;
        }
      /* initiate a h-l edge at strobe pin to mark the data valid */
      if (cr_set_outstrb (board, LOW))
        {
	  icp_error (board, "cr_set_outstrb");
	  icp_free (outp_buffer, PAGED);
	  return -EIO;
	}
      if (cr_set_outstrb (board, HIGH))
        {
	  icp_error (board, "cr_set_outstrb");
	  icp_free (outp_buffer, PAGED);
	  return -EIO;
	}
    }
  icp_free (outp_buffer, PAGED);

  return outp_index;
}

int
icp_iop_init (int board)
{

  ICP_PRINTF("icpci[%c]: iop_init.\n", 'F');

  icp.read_lock[board] = FALSE;
  icp.write_lock[board] = FALSE;
  icp.inp_buffer[board] = NULL;
  icp.inp_count[board] = 0;
  icp.inp_index[board] = 0;
  
  return OK;
}

